@@ -1,58 +1,59 @@
 /* ====================================================================
- * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
+ *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 /*
@@ -140,7 +141,6 @@ static const command_rec negotiation_cmds[] =
 typedef struct accept_rec {
     char *name;                 /* MUST be lowercase */
     float quality;
-    float max_bytes;
     float level;
     char *charset;              /* for content-type only */
 } accept_rec;
@@ -315,7 +315,6 @@ static const char *get_entry(pool *p, accept_rec *result,
                              const char *accept_line)
 {
     result->quality = 1.0f;
-    result->max_bytes = 0.0f;
     result->level = 0.0f;
     result->charset = "";
 
@@ -392,10 +391,6 @@ static const char *get_entry(pool *p, accept_rec *result,
             && (parm[1] == '\0' || (parm[1] == 's' && parm[2] == '\0'))) {
             result->quality = atof(cp);
         }
-        else if (parm[0] == 'm' && parm[1] == 'x' &&
-                 parm[2] == 'b' && parm[3] == '\0') {
-            result->max_bytes = atof(cp);
-        }
         else if (parm[0] == 'l' && !strcmp(&parm[1], "evel")) {
             result->level = atof(cp);
         }
@@ -613,7 +608,6 @@ static void maybe_add_default_accepts(negotiation_state *neg,
         new_accept->name = "*/*";
         new_accept->quality = 1.0f;
         new_accept->level = 0.0f;
-        new_accept->max_bytes = 0.0f;
     }    
 
     new_accept = (accept_rec *) ap_push_array(neg->accepts);
@@ -626,7 +620,6 @@ static void maybe_add_default_accepts(negotiation_state *neg,
         new_accept->quality = prefer_scripts ? 2.0f : 0.001f;
     }
     new_accept->level = 0.0f;
-    new_accept->max_bytes = 0.0f;
 }
 
 /*****************************************************************
@@ -902,6 +895,7 @@ static int read_types_multi(negotiation_state *neg)
     struct var_rec mime_info;
     struct accept_rec accept_info;
     void *new_var;
+    struct { int any, all; } forbidden;
 
     clean_var_rec(&mime_info);
 
@@ -924,6 +918,9 @@ static int read_types_multi(negotiation_state *neg)
         return HTTP_FORBIDDEN;
     }
 
+    forbidden.any = 0;
+    forbidden.all = 1;
+
     while ((dir_entry = readdir(dirp))) {
         request_rec *sub_req;
 
@@ -951,6 +948,13 @@ static int read_types_multi(negotiation_state *neg)
             sub_req->content_type = CGI_MAGIC_TYPE;
         }
 
+        /* HTTP_FORBIDDEN is returned, e.g., if the path length limit was exceeded */
+        /* HTTP_OK does NOT necessarily mean that the file is really readable! */
+        if (sub_req->status == HTTP_OK)
+            forbidden.all = 0;
+        else if (sub_req->status == HTTP_FORBIDDEN)
+            forbidden.any = 1;
+
         if (sub_req->status != HTTP_OK || !sub_req->content_type) {
             ap_destroy_sub_req(sub_req);
             continue;
@@ -997,6 +1001,10 @@ static int read_types_multi(negotiation_state *neg)
 
     ap_pclosedir(neg->pool, dirp);
 
+    /* If all variants we considered turn out to be forbidden, then return FORBIDDEN */
+    if (forbidden.any && forbidden.all)
+        return HTTP_FORBIDDEN;
+
     set_vlist_validator(r, r);
 
     /* Sort the variants into a canonical order.  The negotiation
@@ -1520,13 +1528,6 @@ static void set_accept_quality(negotiation_state *neg, var_rec *variant)
             }
         }
 
-        /* Check maxbytes -- not in HTTP/1.1 or TCN */
-
-        if (type->max_bytes > 0
-            && (find_content_length(neg, variant) > type->max_bytes)) {
-            continue;
-        }
-
         /* If we are allowed to mess with the q-values
          * and have no explicit q= parameters in the accept header,
          * make wildcards very low, so we have a low chance
@@ -2187,7 +2188,7 @@ static void set_neg_headers(request_rec *r, negotiation_state *neg,
          * content-length, which currently leads to the correct result).
          */
         if (!(variant->sub_req && variant->sub_req->handler)
-            && (len = find_content_length(neg, variant)) != 0) {
+            && (len = (long)find_content_length(neg, variant)) != 0) {
 
             lenstr = (char *) ap_palloc(r->pool, 22);
             ap_snprintf(lenstr, 22, "%ld", len);
@@ -2206,14 +2207,6 @@ static void set_neg_headers(request_rec *r, negotiation_state *neg,
                         ap_array_pstrcat(r->pool, arr, '\0'));
     } 
 
-    /* Theoretically the negotiation result _always_ has a dependence on
-     * the contents of the Accept header because we do 'mxb='
-     * processing in set_accept_quality().  However, variations in mxb
-     * only affect the relative quality of several acceptable variants,
-     * so there is no reason to worry about an unacceptable variant
-     * being mistakenly prioritized.  We therefore ignore mxb in deciding
-     * whether or not to include Accept in the Vary field value.
-     */
     if (neg->is_transparent || vary_by_type || vary_by_language ||
         vary_by_language || vary_by_charset || vary_by_encoding) {
 
