@@ -6,6 +6,7 @@
 #include <stdlib.h>
 
 #include "hsregex.h"
+#include "ap_ctype.h"
 #include "utils.h"
 #include "regex2.h"
 
@@ -82,6 +83,7 @@ static int never = 0;		/* for use in asserts; shuts lint up */
  = #define	REG_PEND	0040
  = #define	REG_DUMP	0200
  */
+ap_private_extern
 API_EXPORT(int)			/* 0 success, otherwise REG_something */
 regcomp(preg, pattern, cflags)
 regex_t *preg;
@@ -98,6 +100,29 @@ int cflags;
 #else
 #	define	GOODFLAGS(f)	((f)&~REG_DUMP)
 #endif
+#ifdef CHARSET_EBCDIC /* Added for Apache by <martin@apache.org> */
+	static int initialized = 0;
+
+	if (!initialized) {
+		unsigned ch, idx = 0;
+		static unsigned char ctlchars_ebcdic[256+1];
+
+		for (ch = 1; ch <= 0xFF; ++ch) {
+			if (ap_iscntrl(ch)) {
+				ctlchars_ebcdic[idx++] = ch;
+			}
+		}
+		ctlchars_ebcdic[idx++] = '\0'; /* redundant */
+
+		for (idx=0; idx < sizeof(cclasses) / sizeof(cclasses[0]); ++idx) {
+			if (strcmp(cclasses[idx].name, "cntrl") == 0) {
+				cclasses[idx].chars = (char *)ctlchars_ebcdic;
+				break;
+			}
+		}
+		initialized = 1;
+	}
+#endif /*CHARSET_EBCDIC*/
 
 	cflags = GOODFLAGS(cflags);
 	if ((cflags&REG_EXTENDED) && (cflags&REG_NOSPEC))
@@ -310,7 +335,7 @@ register struct parse *p;
 		ordinary(p, c);
 		break;
 	case '{':		/* okay as ordinary except if digit follows */
-		REQUIRE(!MORE() || !isdigit(PEEK()), REG_BADRPT);
+		REQUIRE(!MORE() || !ap_isdigit(PEEK()), REG_BADRPT);
 		/* FALLTHROUGH */
 	default:
 		ordinary(p, c);
@@ -322,7 +347,7 @@ register struct parse *p;
 	c = PEEK();
 	/* we call { a repetition if followed by a digit */
 	if (!( c == '*' || c == '+' || c == '?' ||
-				(c == '{' && MORE2() && isdigit(PEEK2())) ))
+				(c == '{' && MORE2() && ap_isdigit(PEEK2())) ))
 		return;		/* no repetition, we're done */
 	NEXT1();
 
@@ -351,7 +376,7 @@ register struct parse *p;
 	case '{':
 		count = p_count(p);
 		if (EAT(',')) {
-			if (isdigit(PEEK())) {
+			if (ap_isdigit(PEEK())) {
 				count2 = p_count(p);
 				REQUIRE(count <= count2, REG_BADBR);
 			} else		/* single number with comma */
@@ -372,7 +397,7 @@ register struct parse *p;
 		return;
 	c = PEEK();
 	if (!( c == '*' || c == '+' || c == '?' ||
-				(c == '{' && MORE2() && isdigit(PEEK2())) ) )
+				(c == '{' && MORE2() && ap_isdigit(PEEK2())) ) )
 		return;
 	SETERROR(REG_BADRPT);
 }
@@ -529,7 +554,7 @@ int starordinary;		/* is a leading * an ordinary character? */
 	} else if (EATTWO('\\', '{')) {
 		count = p_count(p);
 		if (EAT(',')) {
-			if (MORE() && isdigit(PEEK())) {
+			if (MORE() && ap_isdigit(PEEK())) {
 				count2 = p_count(p);
 				REQUIRE(count <= count2, REG_BADBR);
 			} else		/* single number with comma */
@@ -560,7 +585,7 @@ register struct parse *p;
 	register int count = 0;
 	register int ndigits = 0;
 
-	while (MORE() && isdigit(PEEK()) && count <= DUPMAX) {
+	while (MORE() && ap_isdigit(PEEK()) && count <= DUPMAX) {
 		count = count*10 + (GETNEXT() - '0');
 		ndigits++;
 	}
@@ -615,7 +640,7 @@ register struct parse *p;
 		register int ci;
 
 		for (i = p->g->csetsize - 1; i >= 0; i--)
-			if (CHIN(cs, i) && isalpha(i)) {
+			if (CHIN(cs, i) && ap_isalpha(i)) {
 				ci = othercase(i);
 				if (ci != i)
 					CHadd(cs, ci);
@@ -706,8 +731,22 @@ register cset *cs;
 			finish = start;
 /* xxx what about signed chars here... */
 		REQUIRE(start <= finish, REG_ERANGE);
+#ifndef CHARSET_EBCDIC
 		for (i = start; i <= finish; i++)
 			CHadd(cs, i);
+#else /* Added for Apache by <martin@apache.org> */
+		/* Special provision for character ranges [a-zA-Z], */
+		/* which are non-contiguous in EBCDIC: */
+		if ((ap_isupper(start) && ap_isupper(finish)) ||
+		    (ap_islower(start) && ap_islower(finish))) {
+			for (i = start; i <= finish; i++)
+				if (ap_isalpha(i))
+					CHadd(cs, i);
+		} else {
+			for (i = start; i <= finish; i++)
+				CHadd(cs, i);
+		}
+#endif /*CHARSET_EBCDIC*/
 		break;
 	}
 }
@@ -727,7 +766,7 @@ register cset *cs;
 	register char *u;
 	register char c;
 
-	while (MORE() && isalpha(PEEK()))
+	while (MORE() && ap_isalpha(PEEK()))
 		NEXT1();
 	len = p->next - sp;
 	for (cp = cclasses; cp->name != NULL; cp++)
@@ -820,11 +859,11 @@ static char			/* if no counterpart, return ch */
 othercase(ch)
 int ch;
 {
-	assert(isalpha(ch));
-	if (isupper(ch))
-		return(tolower(ch));
-	else if (islower(ch))
-		return(toupper(ch));
+	assert(ap_isalpha(ch));
+	if (ap_isupper(ch))
+		return(ap_tolower(ch));
+	else if (ap_islower(ch))
+		return(ap_toupper(ch));
 	else			/* peculiar, but could happen */
 		return(ch);
 }
@@ -867,7 +906,7 @@ register int ch;
 {
 	register cat_t *cap = p->g->categories;
 
-	if ((p->g->cflags&REG_ICASE) && isalpha(ch) && othercase(ch) != ch)
+	if ((p->g->cflags&REG_ICASE) && ap_isalpha(ch) && othercase(ch) != ch)
 		bothcases(p, ch);
 	else {
 		EMIT(OCHAR, (unsigned char)ch);
