@@ -1,58 +1,59 @@
 /* ====================================================================
- * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
+ *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 /*
@@ -69,6 +70,18 @@
  ***********************************************************************
  *
  *
+ * Error messages in the suexec logfile are prefixed with severity values
+ * similar to those used by the main server:
+ *
+ *  Sev     Meaning
+ * emerg:  Failure of some basic system function
+ * alert:  Bug in the way Apache is communicating with suexec
+ * crit:   Basic information is missing, invalid, or incorrect
+ * error:  Script permission/configuration error
+ * warn:   
+ * notice: Some issue of which the sysadmin/webmaster ought to be aware
+ * info:   Normal activity message
+ * debug:  Self-explanatory
  */
 
 #include "ap_config.h"
@@ -145,6 +158,7 @@ char *safe_env_lst[] =
     "SCRIPT_URL",
     "SERVER_ADMIN",
     "SERVER_NAME",
+    "SERVER_ADDR",
     "SERVER_PORT",
     "SERVER_PROTOCOL",
     "SERVER_SOFTWARE",
@@ -205,7 +219,7 @@ static void clean_env(void)
 
 
     if ((cleanenv = (char **) calloc(AP_ENVBUF, sizeof(char *))) == NULL) {
-        log_err("failed to malloc memory for environment\n");
+        log_err("emerg: failed to malloc memory for environment\n");
 	exit(120);
     }
 
@@ -254,28 +268,67 @@ int main(int argc, char *argv[])
     struct stat dir_info;	/* directory info holder     */
     struct stat prg_info;	/* program info holder       */
 
-    /*
-     * If there are a proper number of arguments, set
-     * all of them to variables.  Otherwise, error out.
-     */
     prog = argv[0];
-    if (argc < 4) {
-	log_err("too few arguments\n");
-	exit(101);
-    }
-    target_uname = argv[1];
-    target_gname = argv[2];
-    cmd = argv[3];
-
     /*
      * Check existence/validity of the UID of the user
      * running this program.  Error out if invalid.
      */
     uid = getuid();
     if ((pw = getpwuid(uid)) == NULL) {
-	log_err("invalid uid: (%ld)\n", uid);
+	log_err("crit: invalid uid: (%ld)\n", uid);
 	exit(102);
     }
+    /*
+     * See if this is a 'how were you compiled' request, and
+     * comply if so.
+     */
+    if ((argc > 1)
+        && (! strcmp(argv[1], "-V"))
+        && ((uid == 0)
+#ifdef _OSD_POSIX
+        /* User name comparisons are case insensitive on BS2000/OSD */
+            || (! strcasecmp(HTTPD_USER, pw->pw_name)))
+#else  /* _OSD_POSIX */
+            || (! strcmp(HTTPD_USER, pw->pw_name)))
+#endif /* _OSD_POSIX */
+        ) {
+#ifdef DOC_ROOT
+        fprintf(stderr, " -D DOC_ROOT=\"%s\"\n", DOC_ROOT);
+#endif
+#ifdef GID_MIN
+        fprintf(stderr, " -D GID_MID=%d\n", GID_MIN);
+#endif
+#ifdef HTTPD_USER
+        fprintf(stderr, " -D HTTPD_USER=\"%s\"\n", HTTPD_USER);
+#endif
+#ifdef LOG_EXEC
+        fprintf(stderr, " -D LOG_EXEC=\"%s\"\n", LOG_EXEC);
+#endif
+#ifdef SAFE_PATH
+        fprintf(stderr, " -D SAFE_PATH=\"%s\"\n", SAFE_PATH);
+#endif
+#ifdef SUEXEC_UMASK
+        fprintf(stderr, " -D SUEXEC_UMASK=%03o\n", SUEXEC_UMASK);
+#endif
+#ifdef UID_MIN
+        fprintf(stderr, " -D UID_MID=%d\n", UID_MIN);
+#endif
+#ifdef USERDIR_SUFFIX
+        fprintf(stderr, " -D USERDIR_SUFFIX=\"%s\"\n", USERDIR_SUFFIX);
+#endif
+        exit(0);
+    }
+    /*
+     * If there are a proper number of arguments, set
+     * all of them to variables.  Otherwise, error out.
+     */
+    if (argc < 4) {
+	log_err("alert: too few arguments\n");
+	exit(101);
+    }
+    target_uname = argv[1];
+    target_gname = argv[2];
+    cmd = argv[3];
 
     /*
      * Check to see if the user running this program
@@ -285,15 +338,17 @@ int main(int argc, char *argv[])
 #ifdef _OSD_POSIX
     /* User name comparisons are case insensitive on BS2000/OSD */
     if (strcasecmp(HTTPD_USER, pw->pw_name)) {
-        log_err("user mismatch (%s instead of %s)\n", pw->pw_name, HTTPD_USER);
+        log_err("crit: calling user mismatch (%s instead of %s)\n",
+		pw->pw_name, HTTPD_USER);
 	exit(103);
     }
-#else  /*_OSD_POSIX*/
+#else  /* _OSD_POSIX */
     if (strcmp(HTTPD_USER, pw->pw_name)) {
-        log_err("user mismatch (%s instead of %s)\n", pw->pw_name, HTTPD_USER);
+        log_err("crit: calling user mismatch (%s instead of %s)\n",
+		pw->pw_name, HTTPD_USER);
 	exit(103);
     }
-#endif /*_OSD_POSIX*/
+#endif /* _OSD_POSIX */
 
     /*
      * Check for a leading '/' (absolute path) in the command to be executed,
@@ -303,7 +358,7 @@ int main(int argc, char *argv[])
      */
     if ((cmd[0] == '/') || (!strncmp(cmd, "../", 3))
 	|| (strstr(cmd, "/../") != NULL)) {
-        log_err("invalid command (%s)\n", cmd);
+        log_err("error: invalid command (%s)\n", cmd);
 	exit(104);
     }
 
@@ -321,7 +376,7 @@ int main(int argc, char *argv[])
      * Error out if the target username is invalid.
      */
     if ((pw = getpwnam(target_uname)) == NULL) {
-	log_err("invalid target user name: (%s)\n", target_uname);
+	log_err("crit: invalid target user name: (%s)\n", target_uname);
 	exit(105);
     }
 
@@ -330,7 +385,7 @@ int main(int argc, char *argv[])
      */
     if (strspn(target_gname, "1234567890") != strlen(target_gname)) {
 	if ((gr = getgrnam(target_gname)) == NULL) {
-	    log_err("invalid target group name: (%s)\n", target_gname);
+	    log_err("crit: invalid target group name: (%s)\n", target_gname);
 	    exit(106);
 	}
 	gid = gr->gr_gid;
@@ -352,7 +407,8 @@ int main(int argc, char *argv[])
 	switch (pid = ufork(target_uname))
 	{
 	case -1:	/* Error */
-	    log_err("failed to setup bs2000 environment for user %s: %s\n",
+	    log_err("emerg: failed to setup bs2000 environment for user "
+		    "%s: %s\n",
 		    target_uname, strerror(errno));
 	    exit(150);
 	case 0:	/* Child */
@@ -361,12 +417,13 @@ int main(int argc, char *argv[])
 	    while (pid != waitpid(pid, &status, 0))
 		;
 	    /* @@@ FIXME: should we deal with STOP signals as well? */
-	    if (WIFSIGNALED(status))
+	    if (WIFSIGNALED(status)) {
 		kill (getpid(), WTERMSIG(status));
+	    }
 	    exit(WEXITSTATUS(status));
 	}
     }
-#endif /*_OSD_POSIX*/
+#endif /* _OSD_POSIX */
 
     /*
      * Save these for later since initgroups will hose the struct
@@ -379,7 +436,7 @@ int main(int argc, char *argv[])
      * Log the transaction here to be sure we have an open log 
      * before we setuid().
      */
-    log_err("uid: (%s/%s) gid: (%s/%s) cmd: %s\n",
+    log_err("info: (target/actual) uid: (%s/%s) gid: (%s/%s) cmd: %s\n",
 	    target_uname, actual_uname,
 	    target_gname, actual_gname,
 	    cmd);
@@ -389,7 +446,7 @@ int main(int argc, char *argv[])
      * a UID less than UID_MIN.  Tsk tsk.
      */
     if ((uid == 0) || (uid < UID_MIN)) {
-	log_err("cannot run as forbidden uid (%d/%s)\n", uid, cmd);
+	log_err("crit: cannot run as forbidden uid (%d/%s)\n", uid, cmd);
 	exit(107);
     }
 
@@ -398,7 +455,7 @@ int main(int argc, char *argv[])
      * or as a GID less than GID_MIN.  Tsk tsk.
      */
     if ((gid == 0) || (gid < GID_MIN)) {
-	log_err("cannot run as forbidden gid (%d/%s)\n", gid, cmd);
+	log_err("crit: cannot run as forbidden gid (%d/%s)\n", gid, cmd);
 	exit(108);
     }
 
@@ -409,7 +466,7 @@ int main(int argc, char *argv[])
      * and setgid() to the target group. If unsuccessful, error out.
      */
     if (((setgid(gid)) != 0) || (initgroups(actual_uname, gid) != 0)) {
-	log_err("failed to setgid (%ld: %s)\n", gid, cmd);
+	log_err("emerg: failed to setgid (%ld: %s)\n", gid, cmd);
 	exit(109);
     }
 
@@ -417,7 +474,7 @@ int main(int argc, char *argv[])
      * setuid() to the target user.  Error out on fail.
      */
     if ((setuid(uid)) != 0) {
-	log_err("failed to setuid (%ld: %s)\n", uid, cmd);
+	log_err("emerg: failed to setuid (%ld: %s)\n", uid, cmd);
 	exit(110);
     }
 
@@ -430,7 +487,7 @@ int main(int argc, char *argv[])
      * directories.  Yuck.
      */
     if (getcwd(cwd, AP_MAXPATH) == NULL) {
-	log_err("cannot get current working directory\n");
+	log_err("emerg: cannot get current working directory\n");
 	exit(111);
     }
 
@@ -439,7 +496,8 @@ int main(int argc, char *argv[])
 	    ((chdir(USERDIR_SUFFIX)) != 0) ||
 	    ((getcwd(dwd, AP_MAXPATH)) == NULL) ||
 	    ((chdir(cwd)) != 0)) {
-	    log_err("cannot get docroot information (%s)\n", target_homedir);
+	    log_err("emerg: cannot get docroot information (%s)\n",
+		    target_homedir);
 	    exit(112);
 	}
     }
@@ -447,13 +505,13 @@ int main(int argc, char *argv[])
 	if (((chdir(DOC_ROOT)) != 0) ||
 	    ((getcwd(dwd, AP_MAXPATH)) == NULL) ||
 	    ((chdir(cwd)) != 0)) {
-	    log_err("cannot get docroot information (%s)\n", DOC_ROOT);
+	    log_err("emerg: cannot get docroot information (%s)\n", DOC_ROOT);
 	    exit(113);
 	}
     }
 
     if ((strncmp(cwd, dwd, strlen(dwd))) != 0) {
-	log_err("command not in docroot (%s/%s)\n", cwd, cmd);
+	log_err("error: command not in docroot (%s/%s)\n", cwd, cmd);
 	exit(114);
     }
 
@@ -461,7 +519,7 @@ int main(int argc, char *argv[])
      * Stat the cwd and verify it is a directory, or error out.
      */
     if (((lstat(cwd, &dir_info)) != 0) || !(S_ISDIR(dir_info.st_mode))) {
-	log_err("cannot stat directory: (%s)\n", cwd);
+	log_err("error: cannot stat directory: (%s)\n", cwd);
 	exit(115);
     }
 
@@ -469,7 +527,7 @@ int main(int argc, char *argv[])
      * Error out if cwd is writable by others.
      */
     if ((dir_info.st_mode & S_IWOTH) || (dir_info.st_mode & S_IWGRP)) {
-	log_err("directory is writable by others: (%s)\n", cwd);
+	log_err("error: directory is writable by others: (%s)\n", cwd);
 	exit(116);
     }
 
@@ -477,7 +535,7 @@ int main(int argc, char *argv[])
      * Error out if we cannot stat the program.
      */
     if (((lstat(cmd, &prg_info)) != 0) || (S_ISLNK(prg_info.st_mode))) {
-	log_err("cannot stat program: (%s)\n", cmd);
+	log_err("error: cannot stat program: (%s)\n", cmd);
 	exit(117);
     }
 
@@ -485,7 +543,7 @@ int main(int argc, char *argv[])
      * Error out if the program is writable by others.
      */
     if ((prg_info.st_mode & S_IWOTH) || (prg_info.st_mode & S_IWGRP)) {
-	log_err("file is writable by others: (%s/%s)\n", cwd, cmd);
+	log_err("error: file is writable by others: (%s/%s)\n", cwd, cmd);
 	exit(118);
     }
 
@@ -493,7 +551,7 @@ int main(int argc, char *argv[])
      * Error out if the file is setuid or setgid.
      */
     if ((prg_info.st_mode & S_ISUID) || (prg_info.st_mode & S_ISGID)) {
-	log_err("file is either setuid or setgid: (%s/%s)\n", cwd, cmd);
+	log_err("error: file is either setuid or setgid: (%s/%s)\n", cwd, cmd);
 	exit(119);
     }
 
@@ -505,7 +563,7 @@ int main(int argc, char *argv[])
 	(gid != dir_info.st_gid) ||
 	(uid != prg_info.st_uid) ||
 	(gid != prg_info.st_gid)) {
-	log_err("target uid/gid (%ld/%ld) mismatch "
+	log_err("error: target uid/gid (%ld/%ld) mismatch "
 		"with directory (%ld/%ld) or program (%ld/%ld)\n",
 		uid, gid,
 		dir_info.st_uid, dir_info.st_gid,
@@ -518,10 +576,20 @@ int main(int argc, char *argv[])
      * "[error] Premature end of script headers: ..."
      */
     if (!(prg_info.st_mode & S_IXUSR)) {
-	log_err("file has no execute permission: (%s/%s)\n", cwd, cmd);
+	log_err("error: file has no execute permission: (%s/%s)\n", cwd, cmd);
 	exit(121);
     }
 
+#ifdef SUEXEC_UMASK
+    /*
+     * umask() uses inverse logic; bits are CLEAR for allowed access.
+     */
+    if ((~SUEXEC_UMASK) & 0022) {
+	log_err("notice: SUEXEC_UMASK of %03o allows "
+		"write permission to group and/or other\n", SUEXEC_UMASK);
+    }
+    umask(SUEXEC_UMASK);
+#endif /* SUEXEC_UMASK */
     clean_env();
 
     /* 
@@ -560,6 +628,6 @@ int main(int argc, char *argv[])
      *
      * Oh well, log the failure and error out.
      */
-    log_err("(%d)%s: exec failed (%s)\n", errno, strerror(errno), cmd);
+    log_err("emerg: (%d)%s: exec failed (%s)\n", errno, strerror(errno), cmd);
     exit(255);
 }
